#include "DeerStudio/EditorEngine/API.h"
#include "DeerStudio/EditorEngine.h"
#include "DeerStudio/EditorEngine/ErrorHandle.h"

#include "angelscript.h"

namespace Deer {
    void EditorEngine::registerEditorEngineFunctions() {
        registerEntityFunctions();
        registerMathFunctions();
        registerUIFunctions();
        registerFrameBufferFunctions();

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void setupAutomaticColumns(int)",
            asFUNCTION(
                setupAutomaticColumns
            ),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void setupColumns(int)",
            asFUNCTION(
                setupColumns
            ),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void endColumns()",
            asFUNCTION(endColumns),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void nextColumn()",
            asFUNCTION(nextColumn),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void print(const string& in)",
            asFUNCTION (print),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "int getResourceCount(ResourceType, const string& in)",
            asFUNCTION(getResourceCount),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getResourceNameById(ResourceType, const string& in, int)",
            asFUNCTION(getResourceNameById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getResourcePathById(ResourceType, const string& in, int)",
            asFUNCTION(getResourcePathById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "int getDirCount(ResourceType, const string& in)",
            asFUNCTION(getDirCount),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getDirPathById(ResourceType, const string& in, int)",
            asFUNCTION(getDirPathById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "string getDirNameById(ResourceType, const string& in, int)",
            asFUNCTION(getDirNameById),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void treeNodeLeaf(const string& in, bool)",
            asFUNCTION(treeNode),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool treeNode(const string& in, bool, any@+, ReciverFunc@+)",
            asFUNCTION(treeNodeRecursive),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool componentNode(const string& in, any@+, ReciverFunc@+)",
            asFUNCTION(componentNode),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "bool componentNode_contextMenu(const string& in, any@+, ReciverFunc@+, ReciverFunc@+)",
            asFUNCTION(componentNode_contextMenu),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void contextItemPopup(const string& in, any@+, ReciverFunc@+)",
            asFUNCTION(contextItemPopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void contextMenuPopup(const string& in, any@+, ReciverFunc@+)",
            asFUNCTION(contextMenuPopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void modalPopup(const string& in, ReciverFunc@+)",
            asFUNCTION(modalPopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void simplePopup(const string& in, ReciverFunc@+)",
            asFUNCTION(simplePopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void openPopup(const string& in, any@)",
            asFUNCTION(openPopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void closePopup()",
            asFUNCTION(closePopup),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void dragDropSource(const string& in, any@, const string& in)",
            asFUNCTION(dragDropSource),
            asCALL_CDECL
        ));

        AS_CHECK(scriptEngine->RegisterGlobalFunction(
            "void dragDropTarget(const string& in, any@+, TransferFunc@+)",
            asFUNCTION(dragDropTarget),
            asCALL_CDECL
        ));


    }
}